/*******************************************************************/  
/*                      ADOBE CONFIDENTIAL                         */
/*                   _ _ _ _ _ _ _ _ _ _ _ _ _                     */
/*                                                                 */
/* Copyright 1992-2008 Adobe Systems Incorporated                  */
/* All Rights Reserved.                                            */
/*                                                                 */
/* NOTICE:  All information contained herein is, and remains the   */
/* property of Adobe Systems Incorporated and its suppliers, if    */
/* any.  The intellectual and technical concepts contained         */
/* herein are proprietary to Adobe Systems Incorporated and its    */
/* suppliers and may be covered by U.S. and Foreign Patents,       */
/* patents in process, and are protected by trade secret or        */
/* copyright law.  Dissemination of this information or            */
/* reproduction of this material is strictly forbidden unless      */
/* prior written permission is obtained from Adobe Systems         */
/* Incorporated.                                                   */
/*                                                                 */
/*******************************************************************/
/*
	An Adobe Premiere Pro video filter (not for 6.x use).  Creates a color
	picker parameter and slider parameter in the effect controls window and
	modifies the source pixels based on the settings.  If the slider is zero,
	the filter adds the RGB values in the color picker to the RGB values of
	each pixel.  If the slider is non-zero, the filter uses the callback
	to get the current frame.  Using the callback for this purpose is purely
	for demonstration purposes.  The current frame is passed in through
	(*theData)->source and using the callback to get the current frame in
	a real filter is only wasting time!

	1/13/2000	Original version - EKS
	10/12/2001  Fixed memory leak - ZAL
	9/16/2002	Moved to C++ - ZAL
	2/26/2003	Updated for Adobe Premiere Pro - ZAL
	7/22/2003	New algorithm, code clean-up - ZAL
	10/19/2009	Ported to 64-bit - ZAL
*/

#include "Simple_Video_Filter.h"

PREMPLUGENTRY DllExport xFilter (short selector, VideoHandle theData)
{
	csSDK_int32		result		= fsUnsupported,
					height		= 0, 
					width		= 0, 
					rowbytes	= 0;

	csSDK_uint32	*srcpix		= 0, 
					*dstpix 	= 0,
					pixelRotate	= 0,
					alpha		= 0,
					redSource	= 0,
					greenSource	= 0,
					blueSource	= 0,
					redAdd		= 0,
					greenAdd	= 0,
					blueAdd		= 0;

	prRect			box;
	PPixHand		callbackPPH	= 0;	

	filterParamsH	paramsH		= 0;

	switch (selector)
	{
		case	fsInitSpec:
			// Since we don't pop a dialog, our response to this
			// "silent setup" selector is the same as our normal
			// setup. Respond to fsInitSpec (if possible) creating
			// a default specsHandle withOUT user intervention.
			result = fsNoErr;
			break;

		// This sample has no setup dialog, and relies on Premiere
		// to draw the UI for the parameters in the Effect Control panel
		case	fsHasSetupDialog:
			result = fsHasNoSetupDialog;
			break;
				
		case	fsSetup:
			result = fsNoErr;

			if ((*theData)->specsHandle)	
			{
				// In a filter that has a need for a more complex setup dialog
				// you would present your platform specific user interface here,
				// storing results in the specsHandle (which you've allocated).
			}
			else
			{
				paramsH = (filterParamsH)((*theData)->piSuites->memFuncs->newHandle)(sizeof(filterParams));

				// Memory allocation failed, no need to continue
				if (!paramsH)
				{
					break;
				}
				// Color param is stored as 0x00BBGGRR,
				// unlike pixels which are 0xAARRGGBB
				(*paramsH)->color = 0x00FF80FF;
				(*paramsH)->slider = 0;
				(*paramsH)->checkbox = 0;

				(*theData)->specsHandle = (char**)paramsH;
			}
			break;
		
		//	NOTE: During fsCanHandlePAR, OR together a combination
		//	of flags (from PrSDKEffect.h). DO NOT return a filter
		//	error code.  This selector is only sent if
		//	notUnityPixelAspectRatio and notAnyPixelAspectRatio
		//	are set in the PiPL (.r file).  This selector allows
		//	the filter to ask for a unity PAR during setup, and
		//	the native source PAR during execute, or vice versa.
		case fsCanHandlePAR:
			result = prEffectCanHandlePAR;
			break;

		// New selector for Premiere Pro: successively called to get pixel formats
		// supported.  Set pixelFormatSupported to the longs (i.e. fourCC's)
		// defined in PrSDKPixelFormat.h
		case fsGetPixelFormatsSupported:
			result = fsNoErr;

			switch((*theData)->pixelFormatIndex)
			{
				case 0:
					(*theData)->pixelFormatSupported =  PrPixelFormat_BGRA_4444_8u;
					break;
			
				/*
				If you had more formats, you would add more cases like this:
				case 1:
					(*theData)->pixelFormatSupported =  PrPixelFormat_BGRA_4444_8u;
					break;
				*/
			
				default:
					result = fsBadFormatIndex;
					break;
			}
			break;

		case fsExecute:
			result = fsNoErr;

			// Get the data from specsHandle
			paramsH = (filterParamsH)(*theData)->specsHandle;

			if (paramsH)
			{
				// Get the frame dimensions
				((*theData)->piSuites->ppixFuncs->ppixGetBounds)((*theData)->destination, &box);

				// If the checkbox is checked or slider is non-zero, use callback to get current frame
				// Only a demonstration!  The current frame is passed in through (*theData)->source
				// and using the callback to get it is only wasting time!
				if (((*paramsH)->checkbox || (*paramsH)->slider) && (*theData)->callBack)
				{
					// If we needed to store a frames worth of pixels we'd allocate like this
					// Only a demonstration!  We don't even use this PPix!
					callbackPPH = (*theData)->piSuites->ppixFuncs->ppixNew(&box);
					(*theData)->callBack((*theData)->part, (*theData)->destination, &box, (char **)(*theData)->privateData);
					(*theData)->piSuites->ppixFuncs->ppixDispose(callbackPPH);
				}

				// Do pixel shifting algorithm
				else
				{
					// Get the color parameter and separate into components
					pixelRotate = (long) (*paramsH)->color;
					redAdd = pixelRotate & 0x000000ff;
					greenAdd = (pixelRotate & 0x0000ff00) >> 8;
					blueAdd = (pixelRotate & 0x00ff0000) >> 16;
					
					// Calculate dimensions
					height 		= box.bottom - box.top;
					width 		= box.right - box.left;
					rowbytes	= ((*theData)->piSuites->ppixFuncs->ppixGetRowbytes)((*theData)->destination);

					// Create copies of pointer to the source, destination frames
					srcpix = (csSDK_uint32*)((*theData)->piSuites->ppixFuncs->ppixGetPixels)((*theData)->source);
					dstpix = (csSDK_uint32*)((*theData)->piSuites->ppixFuncs->ppixGetPixels)((*theData)->destination);

					// Loop through each pixel and perform the following:
					// Add B value of color param to B value of pixel
					// Add G value of color param to G value of pixel
					// Add R value of color param to R value of pixel
					// Discard overflow of each addition
					// Do not disturb A
					for(int vert = 0; vert < height; ++vert)
					{
						for(int horiz = 0; horiz < width; ++horiz)
						{
							// Save off alpha
							alpha = *srcpix & 0xff000000;
							// Separate colors
							redSource = (*srcpix & 0x00ff0000) >> 16;
							greenSource = (*srcpix & 0x0000ff00) >> 8;
							blueSource = *srcpix & 0x000000ff;
							// Add param color values and truncate to 8 bits
							redSource = (redSource + redAdd) & 0x000000ff;
							greenSource = (greenSource + greenAdd) & 0x000000ff;
							blueSource = (blueSource + blueAdd) & 0x000000ff;
							// Combine components
							*dstpix = (alpha | (redSource << 16) | (greenSource << 8) | blueSource);

							++dstpix;
							++srcpix;
						}
						
						srcpix += (rowbytes / 4) - width;
						dstpix += (rowbytes / 4) - width;
					} // else
				} // if (((*paramsH)->sliderS) && (*theData)->callBack)

			} // if (specsH)

			// No specsHandle?!  How'd that happen?
			// Oh well, there's nothing we can do at this point.
			else
			{
				result = fsNoErr;
			}
			break;
	}

	// This filter doesn't check for errors.
	// But be sure to return fsUnsupported for unsupported selectors.
	return(result);
}